# -*- coding: utf-8 -*-
"""
Created on Thu Mar 08 20:02:09 2018

@author: sarth
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy import ndimage


def order_disorder_separation(image, percentile, size):
    
    """
    Seperates the input image into order and disorder regions 
    using percentile_filter from scipy.ndimage.
    This function will also provide parameters 
    such as order-diorder ratio, order percentage, disorder percentage 
    and total percent coverage from the separated images.
    
    Args:
        image: segmented image, numpy array
        percentile: float
        size: size of the separation filter, int
    
    Returns:
        filt_img: numpy array 
        I_ordered: float 
        I_disordered: float 
        order_disorder_ratio: float  
        percent_ordered: float  
        percent_disordered: float  
        percent_coverage: float 

    Raises:
        Errors when input datatype is wront
        Error when percentile is out of range.
    
    """

    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # Checking the right data type for the percentile
    assert type(percentile) == int, ('Wrong data type', 'percentile must be an integer')
    
    assert 0 <= percentile <= 100, ('Out of range', 'Percentile value must be between 0 and 100')    

    # Checking the right data type for the size
    assert type(size) == int, ('Wrong data type', 'size must be an integer')
    
    # Using percentile filter to filter image into two labels - 0 and 1 
    filt_img = ndimage.percentile_filter(image, percentile, size, mode='reflect')

    
    # creating a boolean array of all the labels - True = label == 1 and False = label == 0
    q = filt_img == 1
    
    # creating empty arrays similar to input image
    I_ordered = np.zeros_like(image)
    I_disordered = np.zeros_like(image)
    
    # Assigning values for ordered and disordered regions
    I_ordered[q] = image[q]
    I_disordered[~q] = image[~q]
    
    # Plotting Original, Ordered and Disordered Image
    figure, (ax1, ax2, ax3) = plt.subplots(1, 3, figsize=(10, 2.5))
    img1 = ax1.imshow(image, cmap='gray', interpolation='nearest')
    ax1.set_title('Segmented image, before separation')
    plt.colorbar(img1, ax=ax1, shrink=0.5)
    
    img2 = ax2.imshow(I_ordered, cmap='gray', interpolation='nearest')
    ax2.set_title('Ordered region')
    plt.colorbar(img2, ax=ax2, shrink=0.5)

    img3 = ax3.imshow(I_disordered, cmap='gray', interpolation='nearest')
    ax3.set_title('Diordered region')
    plt.colorbar(img3, ax=ax3, shrink=0.5)

    plt.show()

        
    # Calculating order disorder ratio, percent coverage of ordered, disordered and overall
    order_disorder_ratio = np.sum(np.sum(I_ordered)) / np.sum(np.sum(I_disordered))
    
    percent_ordered =100 * (np.sum(np.sum(I_ordered)) / (image.shape[0] * image.shape[1]))
    
    percent_disordered =100 * (np.sum(np.sum(I_disordered)) / (image.shape[0] * image.shape[1]))
    
    percent_coverage = percent_ordered + percent_disordered

    print ('--- Disorderness of Image ---')    
    print ('Order-Disorder ratio = %.5f' %(order_disorder_ratio))
    print ('Order Percentage = %.5f' %(percent_ordered))
    print ('Disorder Percentage = %.5f' %(percent_disordered))
    print ('Coverage Percentage = %.5f' %(percent_coverage))
    
    return filt_img, I_ordered, I_disordered, order_disorder_ratio, percent_ordered, percent_disordered, percent_coverage 






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar 11 15:11:44 2018

@author: demiliu
"""
from ImageSeparationFunc import order_disorder_separation


def sep(segmented_image):
    """
    Wrapper function to perform and optimize segmentation of the image.
    
    Args:
        segmented_image: image to be separated by disorder level,
        numpy array
        
    Returns:
        output from order_disorder_separation
    
    """
    
    #Optimize separation results based on user interaction
    #After each run of the separation function, the user will be asked
    #if separation results look okay. The function will run 
    #until user is okay with the separation.

    sep_ok = 'N'
    while sep_ok == 'N':
        
        #ask user the parameters of the filter
        #if the user chooses to not to use the default setting,
        #the program will ask user to enter parameters
        print ('Please enter the parameters for image separation')
        default_parameter = input('Default parameters? Y/N ')
        
        assert default_parameter == str('Y') or ('N'), ('Answer to "default parameters?" can only be Y or N.')            
        
        if default_parameter == 'Y':            
            percentile = 30
            size = 10
            image_separation = order_disorder_separation(segmented_image, percentile, size)
            sep_ok = input('Are you okay with the separation? Y/N ')

        elif default_parameter == 'N':
            percentile = int(input('Percentile (0 to 100): '))
            size = int(input('Size (integer): '))        
            image_separation = order_disorder_separation(segmented_image, percentile, size)
            sep_ok = input('Are you okay with the separation? Y/N ')
        
        else:
            print ('Please enter Y or N only for "Default parameters?".')
            sep_ok = 'N'

    
    return image_separation


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 07 12:20:10 2018

@author: sarth
"""

import numpy as np
import matplotlib.pyplot as plt
from skimage import segmentation
import cv2


def convert_to_grayscale(image):
    """
    Converting the image to grayscale - 
    where minimum pixel value is 0.0 and maximum pixel value is 1.0
    
    Args:
        image: image to be processed, numpy array
    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
        
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # converting to grayscale
    dst = np.zeros(image.shape)
    image_gray = cv2.normalize(image, dst, 0.0, 1.0, cv2.NORM_MINMAX)
    

    
    return image_gray


def seg_random_walker(image, marker_threshold):
    
    """
    Segment image into two regions using skimage random walker 
    segmentation algorithm. Input image must be gray-scale. 
    This function will provide segmented image and marker positions
    as the output.
    
    Args:
        image: image to be processed, numpy array
        marker_threshold: threshold for segmentation, float
    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
        Error when marker_threshold is out of the range of 0 and 1
    
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong image data type', 'image must be a numpy array')
    
    # Checking that the input image is grayscale and not RGB or something else
    assert np.max(image) == 1.0 or 1, ('Wrong input image type', 'image must be grayscale')
    
    assert np.min(image) == 0.0 or 0, ('Wrong input image type', 'image must be grayscale')
    
    # Checking the right data type and range for marker threshold value
    #assert marker_threshold == float, ('Marker threshold must be a float number.')    
    assert 0.0 <= marker_threshold <= 1.0, ('Marker threshold out of range', 'Marker threshold must be between 0 and 1')    
    
    # Random walker segmentation    
    markers = np.zeros(image.shape)
    markers[image < marker_threshold] = 1 # ordered regions
    markers[image > marker_threshold] = 2 # disordered regions
    
    # Segment image based on order and disorder;
    # convert to gray-scale
    segmented = segmentation.random_walker(image, markers)
    seg_image = convert_to_grayscale(segmented)
    
    
    # Plotting the segemented, original image and markers  
    figure, (ax1, ax2, ax3) = plt.subplots(1, 3, figsize=(10, 2.5))
    img1 = ax1.imshow(image, cmap='gray', interpolation='nearest')
    ax1.set_title('Original Image')
    plt.colorbar(img1, ax=ax1, shrink=0.5)
    
    img2 = ax2.imshow(seg_image, cmap='gray', interpolation='nearest')
    ax2.set_title('Segmented Image')
    plt.colorbar(img2, ax=ax2, shrink=0.5)

    img3 = ax3.imshow(markers, cmap='gray', interpolation='nearest')
    ax3.set_title('Markers')
    plt.colorbar(img3, ax=ax3, shrink=0.5)

    plt.show()

       
    return seg_image
 







# -*- coding: utf-8 -*-
"""
Created on Tue Mar 13 13:30:47 2018

@author: sarth
"""

import matplotlib.pyplot as plt
import pandas as pd
from sklearn.neighbors import KNeighborsClassifier
from sklearn.model_selection import train_test_split
from sklearn import preprocessing
from sklearn import utils
from mpl_toolkits.mplot3d import Axes3D
import numpy as np

pd.set_option('display.float_format', lambda x: '%.6f' % x)

# Reading the DataFile
df = pd.read_csv('afm_datafile_v3.csv')

# Deciding parameters
test_size = 0.2 # Test size
k = 5 # Number of neighbors
weights = 'distance' # uniform or distance based weights for knn

# Concentration is in nM in Original dataset - converting to uM for making better plots
df['concentration'] = df['concentration'] * 1e6


#------------------------------- NO USER INPUT BELOW THIS --------------------------

# Train-Test split
df_train, df_test = train_test_split(df, test_size = test_size, random_state = 15)

# Converting label type from continuous to Multiclass for knn
lab_enc = preprocessing.LabelEncoder()
encoded_ROD = lab_enc.fit_transform(df_train['ROD'])
utils.multiclass.type_of_target(encoded_ROD)

# KNN fit here
knn = KNeighborsClassifier(n_neighbors = k, weights = weights)
knn.fit(df_train[['concentration', 'pH']], encoded_ROD)

#Predict Ratio of order to disorder (ROD) for test data set
testing_pred = knn.predict(df_test[['concentration', 'pH']])

#Display the predictions for Ratio of Order to Disorder for Test set
print ('The disorder value prediction is: ' + str(testing_pred))


#Assign disorder level based on the predicted value above
def disorder_level(disorder):
    level = []
    for i in disorder:
        if i == 0:
            level += ['completely disordered']

        elif 0 < i <= 1:
            level += ['highly disordered']

        elif i > 1:
            level += ['completely ordered']
    
    return level


#Conver disorder number to a categoty
#Printing the predictions for Ratio of Order to Disorder for Test set
disorder = disorder_level(testing_pred)
print ('Disorder level of your sample is: ' + str(disorder))

#disorder level of actual data
actual = disorder_level(np.array(df_test['ROD']))


# Calculating error
correct = 0 
for i in range(len(disorder)):
    if disorder[i] == actual[i]:
        correct += 1
    else:
        correct = correct

#calculating ratio of correct prediction to all predictions
correct_ratio = correct / len(disorder)

print ('The percent of correct prediction is: ' + str(correct_ratio))


# Plotting Training data set wrt to concentration, pH and ROD with 'Kind' as color levels
levels,labels = pd.factorize(df_train.Kind)
y = levels

plt.figure()
ax = Axes3D(plt.gcf())
ax.scatter(df_train.concentration, df_train.pH, df_train.ROD, zdir = 'z', c = y, s = 100, depthshade = False, edgecolor = 'k')
ax.set_xlabel('Peptide Concentration (uM)')
ax.set_ylabel('pH')
ax.set_zlabel('Order-Disorder Ratio')
ax.set_title('Training Data', fontsize = 15)
plt.savefig('Order_Disorder_ratio', dpi = 300)

# Plotting Actual ROD vs Predicted values for specific concentration and pH from KNN for Test data set
plt.figure()
ax = Axes3D(plt.gcf())
ax.scatter(df_test.concentration, df_test.pH, testing_pred, zdir = 'z', c = 'r', marker = '*', s = 70, depthshade = False, label = 'Prediction')
ax.scatter(df_test.concentration, df_test.pH, df_test.ROD, zdir = 'z', c = 'b', marker = 'o', s = 100, depthshade = False, label = 'Actual', edgecolor = 'k')
ax.set_xlabel('Peptide Concentration (uM)')
ax.set_ylabel('pH')
ax.set_zlabel('Order-Disorder Ratio')
ax.set_title('Testing Data vs Prediction', fontsize = 15)
ax.legend(loc = 'best')
plt.savefig('testingdata_vs_prediction', dpi = 300)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar  9 08:56:28 2018

@author: demiliu
"""

import numpy as np
import matplotlib.pyplot as plt
from ImagePreprocessingFunc import convert_to_grayscale

import ImageSegmentation as isg
import ImageSeparation as isp
import ImagePreprocessing as ip



def loadimage(filename):
    """load the user file, .txt format"""    
    txtfile = np.loadtxt(filename, delimiter = '\t')
    
    return txtfile


def savefile(filename, file):
    """saves file"""
    output_file = np.savetxt(filename, file, fmt = '%.3f', delimiter = '\t')
    
    return output_file
    

def savecalculation(filename, file):
    """ saves calculation of disorder-ness:
        1st number: order-disorder ratio
        2nd number: percent ordered
        3rd number: percent disordered
        4th number: percent coverage"""
    output = np.empty((4, 1))
    output[0, 0] = file[0]
    output[1, 0] = file[1]
    output[2, 0] = file[2]
    output[3, 0] = file[3]
    
    output_file = np.savetxt(filename, output, fmt = '%.3f', delimiter = '\t')

    return output_file



#runs one_k() and multiple_k() from command line
if __name__ == '__main__':
    """main function to run functions based on user interaction"""
    
    print ('---AFM Image Segregation---')
    print ('Hello! Welcome to AFM Image Separation!')
    print ('Please enter the filename WITHOUT .txt')
    
    user_filename = input('Filename: ')
    image = loadimage(user_filename + '.txt')


    #display unprocessed image
    plt.figure()
    plt.title('Raw image')
    plt.imshow(image, cmap='gray')
    plt.colorbar()
    plt.show()


    #Perform image processing based on user's need
    print ('How would you like to process the image?')    
    processing_need = input('Please choose from: Background Removal, Segmentation, Separation. ')
    

    if processing_need == 'Background Removal':
        #remove background and save corrected image
        bckgrnd_corr_image = ip.bckgrnd_corr(image)
        bkgnd_savefile = savefile(user_filename + '_bkgn_corr.txt', bckgrnd_corr_image)
        
        #convert image to gray-scale and save gray-scale image
        grayscale_image = convert_to_grayscale(bckgrnd_corr_image)        
        grayscale_savefile = savefile(user_filename + '_bkgn_corr_grayscale.txt', grayscale_image)
        print ('Thanks for using AFM Image Separation. Have a nice day!') 
    
    elif processing_need == 'Segmentation':
        #remove background and save corrected image
        bckgrnd_corr_image = ip.bckgrnd_corr(image)
        bkgnd_savefile = savefile(user_filename + '_bkgn_corr.txt', bckgrnd_corr_image)
        
        #convert image to gray-scale and save gray-scale image
        grayscale_image = convert_to_grayscale(bckgrnd_corr_image)        
        grayscale_savefile = savefile(user_filename + '_bkgn_corr_grayscale.txt', grayscale_image)
        
        #segment image
        segmented_image = isg.seg(grayscale_image)
        segmented_savefile = savefile(user_filename + '_segmented.txt', grayscale_image)
        print ('Thanks for using AFM Image Separation. Have a nice day!') 
    
    
    elif processing_need == 'Separation':
        #remove background and save corrected image
        bckgrnd_corr_image = ip.bckgrnd_corr(image)
        bkgnd_savefile = savefile(user_filename + '_bkgn_corr.txt', bckgrnd_corr_image)
        
        #convert image to gray-scale and save gray-scale image
        grayscale_image = convert_to_grayscale(bckgrnd_corr_image)        
        grayscale_savefile = savefile(user_filename + '_bkgn_corr_grayscale.txt', grayscale_image)
        
        #segment image
        segmented_image = isg.seg(grayscale_image)
        segmented_savefile = savefile(user_filename + '_segmented.txt', segmented_image)

        #separate images
        separated_image = isp.sep(segmented_image)
        filtered_savefile = savefile(user_filename + '_filtered.txt', separated_image[0])
        ordered_savefile = savefile(user_filename + '_ordered.txt', separated_image[1])
        disordered_savefile = savefile(user_filename + '_disordered.txt', separated_image[2])
        calculation_savefile = savecalculation(user_filename + '_disorder_calculation.txt', separated_image[3:])
        print ('Thanks for using AFM Image Separation. Have a nice day!') 
        
    else:
        print ("Sorry! We can't help you...")
        
        exit



# -*- coding: utf-8 -*-
"""
Created on Mon Mar 05 12:15:10 2018

@author: sarth
"""

import numpy as np
import matplotlib.pyplot as plt
from skimage import morphology
import cv2



def savefile(filename, file):
    """ 
    Save an array to a txt file
    
    Args:
        filename: name of the txt file, a string
        file: data to be saved, 1D or 2D array-like
    
    Returns:
        txt file
        
    """

    output_file = np.savetxt(filename, file, fmt = '%.3f', delimiter = '\t')
    
    return output_file



def bckgrnd_correc_rect(image, row_len, col_len):
    """
    Background correction using a rectangular structuring element. 
    This function uses white_tophat from 
    skimage.morphology to return image minus 
    the morphological opening obtained from the structuring element.
    
    Args:
        image: image to be processed, a numpy array
        row_len: side-length of the rectangle filter, int
        col_len: side-length of the rectangle filter, int
    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
    
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # Checking the right data type for the row length of the rectangular structuring element
    assert type(row_len) == int, ('Wrong data type', 'row length must be a float')
    
    # Checking the right data type for the column length of the rectangular structuring element
    assert type(col_len) == int, ('Wrong data type', 'column length must be a float')
    
    # background corrrection 
    image_bckgrnd_corrected = morphology.white_tophat(image, morphology.rectangle(row_len,col_len))
    
    # plotting image
    plt.figure()
    plt.gray()
    plt.title('Background correction with rectangle')
    plt.imshow(image_bckgrnd_corrected)
    plt.colorbar()
    plt.show()
    
    return image_bckgrnd_corrected



def bckgrnd_correc_sq(image, length):
    """
    Background correction using a square structuring element. 
    This function uses white_tophat from 
    skimage.morphology to return image minus 
    the morphological opening obtained from the structuring element.    
    
    Args:
        image: image to be processed, a numpy array
        length: side-length of the square filter, int

    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
    
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # Checking the right data type for the length of the square structuring element
    assert type(length) == int, ('Wrong data type', 'length of the square structuring element must be a float')
    
    # background correction
    image_bckgrnd_corrected = morphology.white_tophat(image, morphology.square(length))
    
    # plotting image
    plt.figure()
    plt.gray()
    plt.title('Background correction with square')
    plt.imshow(image_bckgrnd_corrected)
    plt.colorbar()
    plt.show()
    
    return image_bckgrnd_corrected



def bckgrnd_correc_disk(image, radius):
    """
    Background correction using a disk structuring element. 
    This function uses white_tophat from 
    skimage.morphology to return image minus 
    the morphological opening obtained from the structuring element.    
    
    Args:
        image: image to be processed, a numpy array
        radius: radius of the disk filter, int

    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
    
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # Checking the right data type for the length of the square structuring element
    assert type(radius) == int, ('Wrong data type', 'radius of the disk structuring element must be a float')
    
    # background correction
    image_bckgrnd_corrected = morphology.white_tophat(image, morphology.disk(radius))
    
    # plotting image
    plt.figure()
    plt.gray()
    plt.title('Background correction with disk')
    plt.imshow(image_bckgrnd_corrected)
    plt.colorbar()
    plt.show()
    
    return image_bckgrnd_corrected
        

def convert_to_grayscale(image):
    """
    Converting the image to grayscale - 
    where minimum pixel value is 0.0 and maximum pixel value is 1.0
    
    Args:
        image: image to be processed, numpy array
    
    Returns:
        numpy array
    
    Raises:
        Errors when input type is wrong
        
    """
    
    # Checking the right data type for the input image
    assert type(image) == np.ndarray, ('Wrong data type', 'image must be a numpy array')
    
    # converting to grayscale
    dst = np.zeros(image.shape)
    image_gray = cv2.normalize(image, dst, 0.0, 1.0, cv2.NORM_MINMAX)
    
    # plotting the image
    plt.figure()
    plt.gray()
    plt.title('After background correction, grey-scale')
    plt.imshow(image_gray)
    plt.colorbar()
    plt.show()
    
    return image_gray



def bckgrnd_corr(image):
    """
    Wrapper function to perform and optimize background correction
    
    Args:
        image: numpy array
    
    Returns:
        numpy array
        
    """

    #Optimize background removal results based on user interaction
    #After each run of the function, the user will be asked
    #if background removal results look okay. The function will run 
    #until user is okay with the result.
    
    bkgnd_ok = 'N'
    while bkgnd_ok == 'N':
        
        #user chooses the filter shape
        print ('Please choose filter: Rectangle, Square or Disk.')
        filter_shape = input('Filter: ')

        #perform background correction using a rectangle filter         
        if filter_shape == 'Rectangle':
            
            #ask user the dimension of the filter
            #if the user chooses to not to use the default dimension,
            #the program will ask user to enter dimensions
            print ('What are the dimensions of the filter?')
            default_dim = input('Default dimensions? Y/N ')
            
    
            if default_dim == 'Y':           
                row_len = 10
                col_len = 200
                image_bckgrnd_corrected = bckgrnd_correc_rect(image, row_len, col_len)
                bkgnd_ok = input('Are you okay with background removal?')

            elif default_dim == 'N':
                row_len = int(input('Row length (in pixel): '))
                col_len = int(input('Column length (in pixel): '))
                
                image_bckgrnd_corrected = bckgrnd_correc_rect(image, row_len, col_len)
                bkgnd_ok = input('Are you okay with background removal?')
            
            #if the user did not enter Y or N,
            #a message will show up, and the user will be
            #asked to choose filter shape and dimensions again
            else:
                print ('Please enter Y or N only for "Default dimension?".')
                bkgnd_ok = 'N'

                    
        #perform background correction using a square filter         
        elif filter_shape == 'Square':
            print ('What are the dimensions of the filter?')
            default_dim = input('Default dimensions? Y/N ')

            if default_dim == 'Y':            
                length = 10
                image_bckgrnd_corrected = bckgrnd_correc_sq(image, length)
                bkgnd_ok = input('Are you okay with background removal?')
                
            elif default_dim == 'N':
                length = int(input('Side length (in pixel): '))
                image_bckgrnd_corrected = bckgrnd_correc_sq(image, length)
                bkgnd_ok = input('Are you okay with background removal?')
            
            else:
                print ('Please enter Y or N only for "Default dimension?".')
                bkgnd_ok = 'N'
                

        #perform background correction using a disk filter         
        elif filter_shape == 'Disk':
            print ('What are the dimensions of the filter?')
            default_dim = input('Default dimensions? Y/N ')
            
            if default_dim == 'Y':            
                radius = 5
                image_bckgrnd_corrected = bckgrnd_correc_disk(image, radius)
                bkgnd_ok = input('Are you okay with background removal?')

            elif default_dim == 'N':
                radius = int(input('Radius length (in pixel): '))
                image_bckgrnd_corrected = bckgrnd_correc_disk(image, radius)
                bkgnd_ok = input('Are you okay with background removal?')

            else:
                print ('Please enter Y or N only for "Default dimension?".')
                bkgnd_ok = 'N'
        
        #if user did not enter the given choices,
        #a message will show up. User will be asked to
        #choose shape again.
        else:
            print ('Please choose from the listed shapes.')
        
        
    return image_bckgrnd_corrected
        
        


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 14 19:58:29 2018

@author: sarth
"""

from setuptools import setup

setup(
name = 'SAFMI_AFM',
version = '1.0.1',
packages=['SAFMI/UserImageProcessing', 'SAFMI/ImageDisorderPrediction']
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar 11 15:12:49 2018

@author: demiliu
"""
import numpy as np
import matplotlib.pyplot as plt
from skimage import segmentation
import cv2
from ImageSegmentationFunc import seg_random_walker

def seg(bckgrnd_corr_image):
    """
    Wrapper function to perform and optimize segmentation of the image
    
    Args:
        bckgrnd_corr_image: image to be processed, numpy array
    
    Returns:
        numpy array
    
    """
    
    seg_ok = 'N'
    while seg_ok == 'N':
        print ('Please choose a threshold for segmentation from 0.0 to 1.0')
        default_threshold = input('Default dimensions? Y/N ')
        
        assert default_threshold == 'Y' or 'N', ('Answer to "default threshold?" can only be Y or N.')            
            
        if default_threshold == 'Y':            
            threshold = 0.15
            image_segmentation = seg_random_walker(bckgrnd_corr_image, threshold)
    
        elif default_threshold == 'N':
            threshold = float(input('Threshold (0.0 to 1.0): '))        
            image_segmentation = seg_random_walker(bckgrnd_corr_image, threshold)

            
        seg_ok = input('Are you okay with the segmentation? Y/N ')
    
    return image_segmentation



# -*- coding: utf-8 -*-
"""
Created on Tue Mar 13 13:30:47 2018

@author: sarth
"""

import pandas as pd 
from sklearn.neighbors import KNeighborsClassifier
from sklearn import preprocessing
from sklearn import utils


pd.set_option('display.float_format', lambda x: '%.6f' % x)


def knn_predict(k, concentration, pH):
    """
    Predict disorder by training a knn algorithm with
    existing peptide data. Prediction is based on the set
    of pH and concentration the user provides. 
    
    Args:
        k: int, number of neighbors
        concentration: a float or int, in uM
        pH: a float or int
    
    Return:
        Disorder
    
    Raises:
        Error when the input is out of range
        
    """
    
    assert 1 < k < 50, 'k must an integer within the given range'
    
    assert 0.1 > float(concentration) or float(concentration) < 2.0, 'Concentration is out of range.'
    
    assert 1.0 < float(pH) < 14.0, 'pH out of range.'
    
    # Reading the DataFile containing training and testing set
    df = pd.read_csv('afm_datafile_v3.csv')

    # Deciding parametes
    weights = 'distance' # uniform or distance based weights for knn

    # Create a dataframe based on user inputs
    X_dict = {'concentration': [concentration], 'pH': [pH]}
    predict_X = pd.DataFrame(X_dict)


    # Converting label type from continous to Multiclass for knn
    lab_enc = preprocessing.LabelEncoder()
    encoded_ROD = lab_enc.fit_transform(df['ROD'])
    utils.multiclass.type_of_target(encoded_ROD)

    # KNN fit here
    knn = KNeighborsClassifier(n_neighbors = k, weights = weights)
    knn.fit(df[['concentration', 'pH']], encoded_ROD)

    # Predicting Ratio of order to disorder (ROD) for test data set
    testing_pred = knn.predict(predict_X[['concentration', 'pH']])
    
    if testing_pred == 0:
        disorder_level = 'completely disordered'
    
    elif 0 < testing_pred < 1:
        disorder_level = 'highly disordered'
        
    elif testing_pred > 1:
        disorder_level = 'completely ordered'
        
    #Printing the predictions for Ratio of Order to Disorder for Test set 
    print ('Disorder level of your sample is: ' + disorder_level)
    
    return testing_pred




if __name__ == '__main__':
    """main function to run functions based on user interaction"""
    
    print ('---AFM Peptide Image Disorder Prediction---')
    print ('Hello! Welcome to AFM Peptide Image Disorder Prediction!')
    
    #asks for user's pH and concentration
    print('Please enter a pH value from 1 to 14')
    pH_user = input('pH: ')

    print('Please enter concentration from 0.1 to 2 (uM)')
    conc_user = input('Concentration: ')

    print('Please enter the number of neighbors to compare.')
    k_user = int(input('k (must be integer from 1 to 50): '))
    

    knn_predict(k_user, conc_user, pH_user)
    
    